@@ -7,31 +7,31 @@ from functools import wraps
 
 import numpy as np
 
-from manimlib.constants import *
-from manimlib.utils.color import color_gradient
-from manimlib.utils.color import get_colormap_list
-from manimlib.utils.color import rgb_to_hex
-from manimlib.utils.color import color_to_rgb
-from manimlib.utils.config_ops import digest_config
-from manimlib.utils.iterables import batch_by_property
-from manimlib.utils.iterables import list_update
-from manimlib.utils.iterables import resize_array
-from manimlib.utils.iterables import resize_preserving_order
-from manimlib.utils.iterables import resize_with_interpolation
-from manimlib.utils.iterables import make_even
-from manimlib.utils.iterables import listify
-from manimlib.utils.bezier import interpolate
-from manimlib.utils.bezier import integer_interpolate
-from manimlib.utils.paths import straight_path
-from manimlib.utils.simple_functions import get_parameters
-from manimlib.utils.space_ops import angle_of_vector
-from manimlib.utils.space_ops import get_norm
-from manimlib.utils.space_ops import rotation_matrix_transpose
-from manimlib.shader_wrapper import ShaderWrapper
-from manimlib.shader_wrapper import get_colormap_code
-from manimlib.event_handler import EVENT_DISPATCHER
-from manimlib.event_handler.event_listner import EventListner
-from manimlib.event_handler.event_type import EventType
+from ..constants import *
+from ..utils.color import color_gradient
+from ..utils.color import get_colormap_list
+from ..utils.color import rgb_to_hex
+from ..utils.color import color_to_rgb
+from ..utils.config_ops import digest_config
+from ..utils.iterables import batch_by_property
+from ..utils.iterables import list_update
+from ..utils.iterables import resize_array
+from ..utils.iterables import resize_preserving_order
+from ..utils.iterables import resize_with_interpolation
+from ..utils.iterables import make_even
+from ..utils.iterables import listify
+from ..utils.bezier import interpolate
+from ..utils.bezier import integer_interpolate
+from ..utils.paths import straight_path
+from ..utils.simple_functions import get_parameters
+from ..utils.space_ops import angle_of_vector
+from ..utils.space_ops import get_norm
+from ..utils.space_ops import rotation_matrix_transpose
+from ..shader_wrapper import ShaderWrapper
+from ..shader_wrapper import get_colormap_code
+from ..event_handler import EVENT_DISPATCHER
+from ..event_handler.event_listner import EventListner
+from ..event_handler.event_type import EventType
 
 
 class Mobject(object):
@@ -365,14 +365,17 @@ class Mobject(object):
         self.center()
         return self
 
+    def replicate(self, n):
+        return self.get_group_class()(
+            *(self.copy() for x in range(n))
+        )
+
     def get_grid(self, n_rows, n_cols, height=None, **kwargs):
         """
         Returns a new mobject containing multiple copies of this one
         arranged in a grid
         """
-        grid = self.get_group_class()(
-            *(self.copy() for n in range(n_rows * n_cols))
-        )
+        grid = self.replicate(n_rows * n_cols)
         grid.arrange_in_grid(n_rows, n_cols, **kwargs)
         if height is not None:
             grid.set_height(height)
@@ -408,8 +411,10 @@ class Mobject(object):
         for key in self.data:
             copy_mobject.data[key] = self.data[key].copy()
 
-        # TODO, are uniforms ever numpy arrays?
         copy_mobject.uniforms = dict(self.uniforms)
+        for key in self.uniforms:
+            if isinstance(self.uniforms[key], np.ndarray):
+                copy_mobject.uniforms[key] = self.uniforms[key].copy()
 
         copy_mobject.submobjects = []
         copy_mobject.add(*[sm.copy() for sm in self.submobjects])
@@ -561,7 +566,7 @@ class Mobject(object):
         )
         return self
 
-    def scale(self, scale_factor, min_scale_factor=1e-8, **kwargs):
+    def scale(self, scale_factor, min_scale_factor=1e-8, about_point=None, about_edge=ORIGIN):
         """
         Default behavior is to scale about the center of the mobject.
         The argument about_edge can be a vector, indicating which side of
@@ -574,11 +579,19 @@ class Mobject(object):
         scale_factor = max(scale_factor, min_scale_factor)
         self.apply_points_function(
             lambda points: scale_factor * points,
+            about_point=about_point,
+            about_edge=about_edge,
             works_on_bounding_box=True,
-            **kwargs
         )
+        for mob in self.get_family():
+            mob._handle_scale_side_effects(scale_factor)
         return self
 
+    def _handle_scale_side_effects(self, scale_factor):
+        # In case subclasses, such as DecimalNumber, need to make
+        # any other changes when the size gets altered
+        pass
+
     def stretch(self, factor, dim, **kwargs):
         def func(points):
             points[:, dim] *= factor
@@ -765,6 +778,21 @@ class Mobject(object):
     def set_depth(self, depth, stretch=False, **kwargs):
         return self.rescale_to_fit(depth, 2, stretch=stretch, **kwargs)
 
+    def set_max_width(self, max_width, **kwargs):
+        if self.get_width() > max_width:
+            self.set_width(max_width, **kwargs)
+        return self
+
+    def set_max_height(self, max_height, **kwargs):
+        if self.get_height() > max_height:
+            self.set_height(max_height, **kwargs)
+        return self
+
+    def set_max_depth(self, max_depth, **kwargs):
+        if self.get_depth() > max_depth:
+            self.set_depth(max_depth, **kwargs)
+        return self
+
     def set_coord(self, value, dim, direction=ORIGIN):
         curr = self.get_coord(dim, direction)
         shift_vect = np.zeros(self.dim)
@@ -823,7 +851,6 @@ class Mobject(object):
         return self
 
     def put_start_and_end_on(self, start, end):
-        # TODO, this doesn't currently work in 3d
         curr_start, curr_end = self.get_start_and_end()
         curr_vect = curr_end - curr_start
         if np.all(curr_vect == 0):
@@ -835,9 +862,12 @@ class Mobject(object):
         )
         self.rotate(
             angle_of_vector(target_vect) - angle_of_vector(curr_vect),
-            about_point=curr_start
         )
-        self.shift(start - curr_start)
+        self.rotate(
+            np.arctan2(curr_vect[2], get_norm(curr_vect[:2])) - np.arctan2(target_vect[2], get_norm(target_vect[:2])),
+            axis=np.array([-target_vect[1], target_vect[0], 0]),
+        )
+        self.shift(start - self.get_start())
         return self
 
     # Color functions
@@ -957,7 +987,7 @@ class Mobject(object):
     def add_background_rectangle(self, color=None, opacity=0.75, **kwargs):
         # TODO, this does not behave well when the mobject has points,
         # since it gets displayed on top
-        from manimlib.mobject.shape_matchers import BackgroundRectangle
+        from ..mobject.shape_matchers import BackgroundRectangle
         self.background_rectangle = BackgroundRectangle(
             self, color=color,
             fill_opacity=opacity,
@@ -1063,14 +1093,16 @@ class Mobject(object):
 
     def get_start(self):
         self.throw_error_if_no_points()
-        return np.array(self.get_points()[0])
+        return self.get_points()[0].copy()
 
     def get_end(self):
         self.throw_error_if_no_points()
-        return np.array(self.get_points()[-1])
+        return self.get_points()[-1].copy()
 
     def get_start_and_end(self):
-        return self.get_start(), self.get_end()
+        self.throw_error_if_no_points()
+        points = self.get_points()
+        return (points[0].copy(), points[-1].copy())
 
     def point_from_proportion(self, alpha):
         points = self.get_points()
@@ -1436,7 +1468,7 @@ class Mobject(object):
         return result
 
     def check_data_alignment(self, array, data_key):
-        # Makes sure that self.data[key] can be brodcast into
+        # Makes sure that self.data[key] can be broadcast into
         # the given array, meaning its length has to be either 1
         # or the length of the array
         d_len = len(self.data[data_key])
@@ -1637,7 +1669,7 @@ class _AnimationBuilder:
         return update_target
 
     def build(self):
-        from manimlib.animation.transform import _MethodAnimation
+        from ..animation.transform import _MethodAnimation
 
         if self.overridden_animation:
             return self.overridden_animation
