@@ -1,10 +1,10 @@
 import numpy as np
 import moderngl
 
-from manimlib.constants import GREY_C
-from manimlib.constants import ORIGIN
-from manimlib.mobject.types.point_cloud_mobject import PMobject
-from manimlib.utils.iterables import resize_preserving_order
+from ...constants import GREY_C
+from ...constants import ORIGIN
+from ...mobject.types.point_cloud_mobject import PMobject
+from ...utils.iterables import resize_preserving_order
 
 
 DEFAULT_DOT_RADIUS = 0.05
@@ -68,7 +68,9 @@ class DotCloud(PMobject):
         return self
 
     def set_radii(self, radii):
-        self.data["radii"][:] = resize_preserving_order(radii, len(self.data["radii"]))
+        n_points = len(self.get_points())
+        radii = np.array(radii).reshape((len(radii), 1))
+        self.data["radii"] = resize_preserving_order(radii, n_points)
         self.refresh_bounding_box()
         return self
 
