@@ -1,9 +1,9 @@
 import numpy as np
 from PIL import Image
 
-from manimlib.utils.file_ops import find_file
-from manimlib.utils.directories import get_raster_image_dir
-from manimlib.utils.directories import get_vector_image_dir
+from ..utils.file_ops import find_file
+from ..utils.directories import get_raster_image_dir
+from ..utils.directories import get_vector_image_dir
 
 
 def get_full_raster_image_path(image_file_name):
