@@ -1,10 +1,10 @@
 import numpy as np
 import math
 
-from manimlib.constants import OUT
-from manimlib.utils.bezier import interpolate
-from manimlib.utils.space_ops import get_norm
-from manimlib.utils.space_ops import rotation_matrix_transpose
+from ..constants import OUT
+from ..utils.bezier import interpolate
+from ..utils.space_ops import get_norm
+from ..utils.space_ops import rotation_matrix_transpose
 
 STRAIGHT_PATH_THRESHOLD = 0.01
 
